@@ -32,6 +32,11 @@ const (
 	kilobytes              = 1024
 )
 
+const (
+	metadataDirectiveCopy    = "COPY"
+	metadataDirectiveReplace = "REPLACE"
+)
+
 var copyHelpTemplate = `Name:
 	{{.HelpName}} - {{.Usage}}
 
@@ -107,12 +112,12 @@ Examples:
 
 	22. Upload a file to S3 with a content-type and content-encoding header
 		 > s5cmd --content-type "text/css" --content-encoding "br" myfile.css.br s3://bucket/
-		 
+
 	23. Download the specific version of a remote object to working directory
 		 > s5cmd {{.HelpName}} --version-id VERSION_ID s3://bucket/prefix/object .
 
-	24. Pass arbitrary metadata to the object during upload or copy 
-		 > s5cmd {{.HelpName}} --metadata "camera=Nixon D750" --metadata "imageSize=6032x4032" flowers.png s3://bucket/prefix/flowers.png 
+	24. Pass arbitrary metadata to the object during upload or copy
+		 > s5cmd {{.HelpName}} --metadata "camera=Nixon D750" --metadata "imageSize=6032x4032" flowers.png s3://bucket/prefix/flowers.png
 `
 
 func NewSharedFlags() []cli.Flag {
@@ -141,6 +146,17 @@ func NewSharedFlags() []cli.Flag {
 			Name:  "metadata",
 			Usage: "set arbitrary metadata for the object, e.g. --metadata 'foo=bar' --metadata 'fizz=buzz'",
 		},
+		&cli.GenericFlag{
+			Name:  "metadata-directive",
+			Usage: "set metadata directive for the object: COPY or REPLACE",
+			Value: &EnumValue{
+				Enum:    []string{metadataDirectiveCopy, metadataDirectiveReplace, ""},
+				Default: "",
+				ConditionFunction: func(str, target string) bool {
+					return strings.EqualFold(target, str)
+				},
+			},
+		},
 		&cli.StringFlag{
 			Name:  "sse",
 			Usage: "perform server side encryption of the data at its destination, e.g. aws:kms",
@@ -305,6 +321,7 @@ type Copy struct {
 	contentEncoding       string
 	contentDisposition    string
 	metadata              map[string]string
+	metadataDirective     string
 	showProgress          bool
 	progressbar           progressbar.ProgressBar
 
@@ -382,6 +399,7 @@ func NewCopy(c *cli.Context, deleteSource bool) (*Copy, error) {
 		contentEncoding:       c.String("content-encoding"),
 		contentDisposition:    c.String("content-disposition"),
 		metadata:              metadata,
+		metadataDirective:     c.String("metadata-directive"),
 		showProgress:          c.Bool("show-progress"),
 		progressbar:           commandProgressBar,
 
@@ -425,7 +443,7 @@ func (c Copy) Run(ctx context.Context) error {
 	var (
 		merrorWaiter  error
 		merrorObjects error
-		errDoneCh     = make(chan bool)
+		errDoneCh     = make(chan struct{})
 	)
 
 	go func() {
@@ -514,10 +532,30 @@ func (c Copy) Run(ctx context.Context) error {
 
 		switch {
 		case srcurl.Type == c.dst.Type: // local->local or remote->remote
+			if c.metadataDirective == "" {
+				// default to COPY
+				c.metadataDirective = metadataDirectiveCopy
+				if c.src.IsRemote() && c.dst.IsRemote() {
+					// default to REPLACE when copying between remote storages
+					c.metadataDirective = metadataDirectiveReplace
+				}
+			}
 			task = c.prepareCopyTask(ctx, srcurl, c.dst, isBatch, c.metadata)
 		case srcurl.IsRemote(): // remote->local
+			if c.metadataDirective != "" {
+				err := fmt.Errorf("metadata directive is not supported for download")
+				merrorObjects = multierror.Append(merrorObjects, err)
+				printError(c.fullCommand, c.op, err)
+				continue
+			}
 			task = c.prepareDownloadTask(ctx, srcurl, c.dst, isBatch)
 		case c.dst.IsRemote(): // local->remote
+			if c.metadataDirective != "" {
+				err := fmt.Errorf("metadata directive is not supported for upload")
+				merrorObjects = multierror.Append(merrorObjects, err)
+				printError(c.fullCommand, c.op, err)
+				continue
+			}
 			task = c.prepareUploadTask(ctx, srcurl, c.dst, isBatch, c.metadata)
 		default:
 			panic("unexpected src-dst pair")
@@ -612,7 +650,6 @@ func (c Copy) doDownload(ctx context.Context, srcurl *url.URL, dsturl *url.URL)
 
 	err = c.shouldOverride(ctx, srcurl, dsturl)
 	if err != nil {
-		// FIXME(ig): rename
 		if errorpkg.IsWarning(err) {
 			printDebug(c.op, err, srcurl, dsturl)
 			return nil
@@ -765,6 +802,7 @@ func (c Copy) doCopy(ctx context.Context, srcurl, dsturl *url.URL, extradata map
 		ContentDisposition: c.contentDisposition,
 		EncryptionMethod:   c.encryptionMethod,
 		EncryptionKeyID:    c.encryptionKeyID,
+		Directive:          c.metadataDirective,
 	}
 
 	err = c.shouldOverride(ctx, srcurl, dsturl)
