@@ -70,10 +70,6 @@ var (
 	s5cmdPath        string
 )
 
-func init() {
-	rand.Seed(time.Now().UnixNano())
-}
-
 type setupOpts struct {
 	s3backend   string
 	endpointURL string
@@ -144,7 +140,7 @@ func setup(t *testing.T, options ...option) (*s3.S3, func(...string) icmd.Cmd) {
 	for _, option := range options {
 		option(opts)
 	}
-	testdir, workdir := workdir(t, opts)
+	testdir, workdir := workdir(t)
 
 	endpoint := ""
 
@@ -193,7 +189,7 @@ func setup(t *testing.T, options ...option) (*s3.S3, func(...string) icmd.Cmd) {
 	return client, s5cmd(workdir, endpoint)
 }
 
-func workdir(t *testing.T, opts *setupOpts) (*fs.Dir, string) {
+func workdir(t *testing.T) (*fs.Dir, string) {
 	// testdir := fs.NewDir() tries to create a new directory which has a
 	// prefix = [test function name][operation name]
 	// e.g., prefix' = "TestCopySingleS3ObjectToLocal/cp_s3://bucket/object_file"
@@ -360,6 +356,8 @@ func s5cmd(workdir, endpoint string) func(args ...string) icmd.Cmd {
 		)
 		cmd.Env = env
 		cmd.Dir = workdir
+		// increase timeout for slow CI
+		cmd.Timeout = 10 * time.Minute
 		return cmd
 	}
 }
@@ -707,7 +705,12 @@ func ensureS3Object(
 	}
 
 	if opts.storageClass != nil {
-		if diff := cmp.Diff(opts.storageClass, output.StorageClass); diff != "" {
+		storageClassofOutput := aws.String("STANDARD")
+		if output.StorageClass != nil {
+			storageClassofOutput = output.StorageClass
+		}
+
+		if diff := cmp.Diff(opts.storageClass, storageClassofOutput); diff != "" {
 			return fmt.Errorf("storage-class of %v/%v: (-want +got):\n%v", bucket, key, diff)
 		}
 	}
@@ -745,6 +748,12 @@ func putArbitraryMetadata(metadata map[string]*string) putOption {
 	}
 }
 
+func putStorageClass(storageClass string) putOption {
+	return func(opts *s3.PutObjectInput) {
+		opts.StorageClass = aws.String(storageClass)
+	}
+}
+
 func putFile(t *testing.T, client *s3.S3, bucket string, filename string, content string, opts ...putOption) {
 	t.Helper()
 	input := &s3.PutObjectInput{
