@@ -18,11 +18,8 @@ import (
 	"github.com/peak/s5cmd/v2/strutil"
 )
 
-var (
-
-	// ErrNoObjectFound indicates there are no objects found from a given directory.
-	ErrNoObjectFound = fmt.Errorf("no object found")
-)
+// ErrNoObjectFound indicates there are no objects found from a given directory.
+var ErrNoObjectFound = fmt.Errorf("no object found")
 
 // ErrGivenObjectNotFound indicates a specified object is not found.
 type ErrGivenObjectNotFound struct {
@@ -231,6 +228,11 @@ type Metadata struct {
 	EncryptionKeyID    string
 
 	UserDefined map[string]string
+
+	// MetadataDirective is used to specify whether the metadata is copied from
+	// the source object or replaced with metadata provided when copying S3
+	// objects. If MetadataDirective is not set, it defaults to "COPY".
+	Directive string
 }
 
 func (o Object) ToBytes() []byte {
